#include <stdio.h>
#include <stdlib.h>
#include <wchar.h>
#include <iiimcf.h>
#include "components.h"
#include "util.h"

#include "im.h" /* to use globals */
#include "handler.h" /* definitions of handler functions */

static IIIMF_status
iiimcf_preedit_handler(IIIMCF_context c,
			IIIMCF_event event,
			IIIMCF_component current,
			IIIMCF_component parent)
{
	IIIMF_status st;
	IIIMCF_text text;
	int cur_pos;

	st = iiimcf_get_preedit_text(c, &text, &cur_pos);

	if(st == IIIMF_STATUS_SUCCESS) {
		char * str;
		str = iiimcf_text_to_utf8(text);
#if DEBUG
		fprintf(stderr, "[DEBUG]PREEDIT:       %s(%d)\n", str, cur_pos);
#endif
		TII_PREEDIT(str);
		free(str);
	}
	else if(st == IIIMF_STATUS_NO_PREEDIT)
#if DEBUG
		fprintf(stderr, "[DEBUG]PREEDIT:       No Preedit\n");
#else
		;
#endif
	else
		return IIIMF_STATUS_FAIL;

	return IIIMF_STATUS_SUCCESS;
}

static IIIMF_status
iiimcf_lookup_choice_handler(IIIMCF_context c,
			IIIMCF_event event,
			IIIMCF_component current,
			IIIMCF_component parent)
{
#if DEBUG
	fprintf(stderr, "[DEBUG]LOOKUP         Invoked\n");
#endif
	/* TII_LOOKUP() */
	return IIIMF_STATUS_SUCCESS;
}


static IIIMF_status
iiimcf_status_handler(IIIMCF_context c,
			IIIMCF_event event,
			IIIMCF_component current,
			IIIMCF_component parent)
{
	IIIMF_status st;
	IIIMCF_text text;

	st = iiimcf_get_status_text(c, &text);
	if(st == IIIMF_STATUS_SUCCESS) {
		char * str;

		str = iiimcf_text_to_utf8(text);
#if DEBUG
		fprintf(stderr, "[DEBUG]STATUS:        %s\n", str);
#endif
		TII_STATUS(str);
		free(str);
	}
	else if(st == IIIMF_STATUS_NO_STATUS_TEXT) {
#if DEBUG
		fprintf(stderr, "[DEBUG]STATUS:        No status text\n");
#endif
	}
	else {
		return IIIMF_STATUS_COMPONENT_INDIFFERENT;
	}

	return IIIMF_STATUS_SUCCESS;
}

static IIIMF_status
iiimcf_keyevent_handler(IIIMCF_context c,
			IIIMCF_event event,
			IIIMCF_component current,
			IIIMCF_component parent)
{
#if DEBUG
	fprintf(stderr, "[DEBUG]KEYEVENT       invoked\n");
#endif
	return IIIMF_STATUS_SUCCESS;
}

static IIIMF_status
iiimcf_trigger_notify_handler(IIIMCF_context c,
			IIIMCF_event event,
			IIIMCF_component current,
			IIIMCF_component parent)
{
	IIIMF_status st;
	int flag;

	st = iiimcf_get_trigger_notify_flag(event, &flag);
	if(st == IIIMF_STATUS_SUCCESS) {
#if DEBUG
		fprintf(stderr, "[DEBUG]TRIGGER NOTIFY %d\n", flag);
#else
		;
#endif
	}
	else {
#if DEBUG
		fprintf(stderr, "[DEBUG]TRIGGER NOTIFY WARNING(%d)\n", (int)st);
#endif
		return IIIMF_STATUS_COMPONENT_INDIFFERENT;
	}

	return IIIMF_STATUS_SUCCESS;
}

static IIIMF_status
iiimcf_ui_handler(IIIMCF_context c,
			IIIMCF_event event,
			IIIMCF_component current,
			IIIMCF_component parent)
{
	IIIMF_status st;
	IIIMCF_event_type et;

	st = iiimcf_get_event_type(event, &et);
	if(st != IIIMF_STATUS_SUCCESS) {
#if DEBUG
		fprintf(stderr, "[DEBUG]UI:            CANT_GET_EVENT_TYPE\n");
#endif
		return IIIMF_STATUS_COMPONENT_INDIFFERENT;
	}

	if(et == IIIMCF_EVENT_TYPE_UI_COMMIT) {
		IIIMCF_text text;

		st = iiimcf_get_committed_text(c, &text);
		if(st == IIIMF_STATUS_SUCCESS) {
			char * str;
			str = iiimcf_text_to_utf8(text);
#if DEBUG
			fprintf(stderr, "[DEBUG]UI-COMMIT:     %s\n", str);
#endif
			TII_COMMIT(str);
			free(str);
		}
		else if(st == IIIMF_STATUS_NO_COMMITTED_TEXT) {
#if DEBUG
			fprintf(stderr, "[DEBUG]UI-COMMIT:     No text\n");
#else
			;
#endif
		}
		else {
			return IIIMF_STATUS_COMPONENT_INDIFFERENT;
		}
		return IIIMF_STATUS_SUCCESS;
	}

	return IIIMF_STATUS_COMPONENT_INDIFFERENT;
}

static IIIMF_status
iiimcf_event_handler(IIIMCF_context c,
			IIIMCF_event event,
			IIIMCF_component current,
			IIIMCF_component parent)
{
#if DEBUG
	fprintf(stderr, "[DEBUG]EVENT          invoked\n");
#endif
	return IIIMF_STATUS_COMPONENT_INDIFFERENT;
}

static IIIMF_status
iiimcf_aux_handler(IIIMCF_context c,
			IIIMCF_event event,
			IIIMCF_component current,
			IIIMCF_component parent)
{
#if DEBUG
	fprintf(stderr, "[DEBUG]AUX            invoked\n");
#endif
	return IIIMF_STATUS_SUCCESS;
}

int
register_iiimcf_components(void)
{
	IIIMF_status st;
	IIIMCF_component parent, comp;
	IIIMCF_handle * handle = global_iiimcf_handle((IIIMCF_handle *)NULL);

	/*
	 * PREEDIT HANDLER
	 */
	st = iiimcf_get_component(*handle,
				"org.OpenI18N.IIIMCF.UI.preedit",
				&parent);
	if(st != IIIMF_STATUS_SUCCESS)
		return 1;
	st = iiimcf_register_component(*handle,
				"net.nskystars.TII.UI.preedit",
				iiimcf_preedit_handler,
				parent,
				&comp);
	if(st != IIIMF_STATUS_SUCCESS)
		return 2;

	/*
	 * LOOKUP CHOICE HANDLER
	 */
	st = iiimcf_get_component(*handle,
				"org.OpenI18N.IIIMCF.UI.lookup_choice",
				&parent);
	if(st != IIIMF_STATUS_SUCCESS)
		return 3;
	st = iiimcf_register_component(*handle,
				"net.nskystars.TII.UI.lookup_choice",
				iiimcf_lookup_choice_handler,
				parent,
				&comp);
	if(st != IIIMF_STATUS_SUCCESS)
		return 4;

	/*
	 * STATUS HANDLER
	 */
	st = iiimcf_get_component(*handle,
				"org.OpenI18N.IIIMCF.UI.status",
				&parent);
	if(st != IIIMF_STATUS_SUCCESS)
		return 5;
	st = iiimcf_register_component(*handle,
				"net.nskystars.TII.UI.status",
				iiimcf_status_handler,
				parent,
				&comp);
	if(st != IIIMF_STATUS_SUCCESS)
		return 6;

	/*
	 * KEYEVENT HANDLER
	 */
	st = iiimcf_get_component(*handle,
				"org.OpenI18N.IIIMCF.event.key",
				&parent);
	if(st != IIIMF_STATUS_SUCCESS)
		return 7;
	st = iiimcf_register_component(*handle,
				"net.nskystars.TII.event.key",
				iiimcf_keyevent_handler,
				parent,
				&comp);
	if(st != IIIMF_STATUS_SUCCESS)
		return 8;

	/*
	 * TRIGGER NOTIFY EVENT HANDLER
	 */
	st = iiimcf_get_component(*handle,
				"org.OpenI18N.IIIMCF.event.trigger_notify",
				&parent);
	if(st != IIIMF_STATUS_SUCCESS)
		return 9;
	st = iiimcf_register_component(*handle,
				"net.nskystars.TII.event.trigger_notify",
				iiimcf_trigger_notify_handler,
				parent,
				&comp);
	if(st != IIIMF_STATUS_SUCCESS)
		return 10;

	/*
	 * UI EVENT HANDLER
	 */
	st = iiimcf_get_component(*handle,
				"org.OpenI18N.IIIMCF.UI",
				&parent);
	if(st != IIIMF_STATUS_SUCCESS)
		return 11;
	st = iiimcf_register_component(*handle,
				"net.nskystars.TII.UI",
				iiimcf_ui_handler,
				parent,
				&comp);
	if(st != IIIMF_STATUS_SUCCESS)
		return 12;

	/*
	 * EVENT EVENT HANDLER
	 */
	st = iiimcf_get_component(*handle,
				"org.OpenI18N.IIIMCF.event",
				&parent);
	if(st != IIIMF_STATUS_SUCCESS)
		return 13;
	st = iiimcf_register_component(*handle,
				"net.nskystars.TII.event",
				iiimcf_event_handler,
				parent,
				&comp);
	if(st != IIIMF_STATUS_SUCCESS)
		return 14;

	/*
	 * AUX EVENT HANDLER
	 */
	st = iiimcf_get_component(*handle,
				"org.OpenI18N.IIIMCF.AUX",
				&parent);
	if(st != IIIMF_STATUS_SUCCESS)
		return 15;
	st = iiimcf_register_component(*handle,
				"net.nskystars.TII.AUX",
				iiimcf_aux_handler,
				parent,
				&comp);
	if(st != IIIMF_STATUS_SUCCESS)
		return 16;

	return 0;
}
