#include <stdio.h>
#include <stdlib.h>
#include <wchar.h>
#include <time.h>
#include <iiimcf.h>
#include "components.h"
#include "im.h"
#include "util.h"

/* tii global */
char *
tii_global_use_im(char * new_im_name)
{
	static char * im_name;
	if(new_im_name != NULL) {
		im_name = new_im_name;
	}
	return im_name;
}

/* tii-iiimcf global */
IIIMCF_handle *
global_iiimcf_handle(IIIMCF_handle * new_handle)
{
	static IIIMCF_handle handle;
	if(new_handle != NULL) {
		handle = *new_handle;
	}
	return &handle;
}

IIIMCF_context *
global_iiimcf_context(IIIMCF_context * new_context)
{
	static IIIMCF_context context;
	if(new_context != NULL) {
		context = *new_context;
	}
	return &context;
}

int
forward_event(IIIMCF_event ev)
{
	IIIMF_status st;
	IIIMCF_context * c = global_iiimcf_context((IIIMCF_context *)NULL);

	st = iiimcf_forward_event(*c, ev); 
	if(st == IIIMF_STATUS_EVENT_NOT_FORWARDED) {
#if DEBUG
		fprintf(stderr, "[DEBUG]Event not forwarded\n");
#else
		;
#endif
	}
	else if(st != IIIMF_STATUS_SUCCESS)
		return 1;

	return 0;
}

int
create_handle(char * client_type)
{
	IIIMF_status st;
	IIIMCF_attr attr;
	IIIMCF_handle * handle = global_iiimcf_handle((IIIMCF_handle *)NULL);

	st = iiimcf_create_attr(&attr);
	if(st != IIIMF_STATUS_SUCCESS)
		return 1;

	st = iiimcf_attr_put_string_value(attr,
					IIIMCF_ATTR_CLIENT_TYPE,
					client_type);
	if(st != IIIMF_STATUS_SUCCESS)
		return 2;

	st = iiimcf_create_handle(attr, handle);
	if(st != IIIMF_STATUS_SUCCESS)
		return 3;

	st = iiimcf_destroy_attr(attr);
	if(st != IIIMF_STATUS_SUCCESS)
		return 4;

	return 0;
}

IIIMCF_input_method
get_input_method_id(void)
{
	char * im_name = tii_global_use_im((char *)NULL);
	IIIMCF_handle * handle = global_iiimcf_handle((IIIMCF_handle *)NULL);
	const IIIMP_card16 *im_id;
	int im_num, i;
	IIIMCF_input_method * input_method;
	IIIMF_status st;

	st = iiimcf_get_supported_input_methods(*handle,
						&im_num,
						&input_method);
	if(st != IIIMF_STATUS_SUCCESS)
		return 0;

	for(i = 0; i < im_num; i++) {
		iiimcf_get_input_method_desc(input_method[i],
						&im_id,
						NULL,
						NULL);
		char * str;
		str = card16_to_utf8(im_id);
		if(!strcmp(im_name, str)) {
#if DEBUG
			fprintf(stderr, "[DEBUG]Using input method: %s\n", str);
#endif
			free(str);
			return input_method[i];
		}
		free(str);
	}

	return 0;
}

int
create_context(IIIMCF_input_method im)
{
	IIIMCF_attr attr;
	IIIMF_status st;
	IIIMCF_handle * handle = global_iiimcf_handle((IIIMCF_handle *)NULL);
	IIIMCF_context * context =
				global_iiimcf_context((IIIMCF_context *)NULL);

	st = iiimcf_create_attr(&attr);
	if(st != IIIMF_STATUS_SUCCESS)
		return 1;

	st = iiimcf_attr_put_ptr_value(attr, IIIMCF_ATTR_INPUT_METHOD, im);
	if(st != IIIMF_STATUS_SUCCESS)
		return 2;

	st = iiimcf_create_context(*handle, attr, context);
	if(st != IIIMF_STATUS_SUCCESS)
		return 3;

	st = iiimcf_destroy_attr(attr);
	if(st != IIIMF_STATUS_SUCCESS)
		return 4;

	return 0;
}

int
forward_im_trigger(int flag)
{
	IIIMCF_event ev;
	IIIMF_status st;

	st = iiimcf_create_trigger_notify_event(flag, &ev);
	if(st != IIIMF_STATUS_SUCCESS)
		return 1;

	if(forward_event(ev))
		return 2;

	return 0;
}

int
forward_key(int keycode, int keychar, int mod)
{
	IIIMF_status st;
	IIIMCF_event ev;
	IIIMCF_keyevent kev;

	kev.keycode = keycode;
	kev.keychar = keychar;
	kev.modifier = mod;
	kev.time_stamp = time(NULL);

	st = iiimcf_create_keyevent(&kev, &ev);
	if(st != IIIMF_STATUS_SUCCESS)
		return 1;

	if(forward_event(ev))
		return 2;

	return 0;
}

int
dispatch_incoming_events(void)
{
	IIIMF_status st;
	IIIMCF_event ev;
	IIIMCF_context * c = global_iiimcf_context((IIIMCF_context *)NULL);

	while((st = iiimcf_get_next_event(*c, &ev)) == IIIMF_STATUS_SUCCESS) {
#if DEBUG
		debug_print_event(ev);
#endif
		st = iiimcf_dispatch_event(*c, ev);
		if(st != IIIMF_STATUS_SUCCESS) {
			if(st == IIIMF_STATUS_COMPONENT_FAIL) {
#if DEBUG
				fprintf(stderr, "[DEBUG]Component failure\n");
#else
				;
#endif
			}
			else if(st == IIIMF_STATUS_COMPONENT_INDIFFERENT) {
#if DEBUG
				fprintf(stderr, "[DEBUG]Component ignored\n");
#else
				;
#endif
			}
			else
				return 1;
		}
		st = iiimcf_ignore_event(ev);
		if(st != IIIMF_STATUS_SUCCESS)
			return 2;
	}
	if(st != IIIMF_STATUS_NO_EVENT)
		return 3;

	return 0;
}
