#include <stdio.h>
#include <stdlib.h>
#include <wchar.h>
#include <iiimcf.h>
#include "util.h"

#define CONVERT_BUFSIZE 48

/* from hangul.c in nabi */
int
wchar_to_utf8(wchar_t c, char * outbuf, int bufsize)
{
	unsigned int len = 0;
	int first, i;

	if(c < 0x80) {
		first = 0;
		len = 1;
	}
	else if(c < 0x800) {
		first = 0xc0;
		len = 2;
	}
	else if(c < 0x10000) {
		first = 0xe0;
		len = 3;
	}
	else if(c < 0x200000) {
		first = 0xf0;
		len = 4;
	}
	else if(c < 0x4000000) {
		first = 0xf8;
		len = 5;
	}
	else {
		first = 0xfc;
		len = 6;
	}

	if(len > bufsize)
		return 0;

	if(outbuf) {
		for(i = len - 1; i > 0; --i) {
			outbuf[i] = (c & 0x3f) | 0x80;
			c >>= 6;
		}
		outbuf[0] = c | first;
	}

	return len;
}

/* from common.c in im-sdk */
char *
iiimcf_text_to_utf8(IIIMCF_text t)
{
	IIIMF_status st;
	char *p, *pr, *pr2;
	int i, j, n, l, rest, nfb, initlen = 256;
	IIIMP_card16 ch;
	const IIIMP_card32 *pids, *pfbs;
	char utf8buf[CONVERT_BUFSIZE] = {0, };

	pr = p = (char *)malloc(sizeof(*p) * initlen);
	rest = initlen - 1;

	st = iiimcf_get_text_length(t, &n);
	if(st != IIIMF_STATUS_SUCCESS)
		return 0;

	for(i = 0; i < n; i++) {
		st = iiimcf_get_char_with_feedback(t, i, &ch, &nfb,
							&pids, &pfbs);
		if(st != IIIMF_STATUS_SUCCESS)
			return 0;

		if(rest <= (16 + 2 + 20 * nfb)) {
			rest += initlen;
			initlen *= 2;
			pr2 = (char *)realloc(pr, sizeof(*p) * initlen);
			p = pr2 + (p - pr);
			pr = pr2;
		}
		if(ch < 128) {
			*p++ = ch;
			rest--;
		}
		else {
			wchar_to_utf8((wchar_t)ch, utf8buf, CONVERT_BUFSIZE);
			l = snprintf(p, CONVERT_BUFSIZE, "%s", utf8buf);
			p += l;
			rest -= l;
		}
	}
	*p = '\0';

	return pr;
}

/* from common.c in im-sdk */
char *
card16_to_utf8(const IIIMP_card16 * pu16)
{
	char *p, *pr, *pr2;
	int i, rest;
	int initlen = 256;
	char utf8buf[CONVERT_BUFSIZE] = {0, };

	pr = (char *)malloc(sizeof(*p) * initlen);
	rest = initlen - 1;
	for(p = pr; *pu16; pu16++) {
		if(*pu16 < 128) {
			if(rest < 1) {
				rest += initlen;
				initlen *= 2;
				pr2 = (char *)realloc(pr, sizeof(*p) * initlen);
				p = pr2 + (p - pr);
				pr = pr2;
			}
			*p++ = *pu16;
			rest--;
		}
		else {
			if(rest < 16) {
				rest += initlen;
				initlen *= 2;
				pr2 = (char *)realloc(pr, sizeof(*p) * initlen);
				p = pr2 + (p - pr);
				pr = pr2;
			}
			wchar_to_utf8((wchar_t)*pu16, utf8buf, CONVERT_BUFSIZE);
			i = snprintf(p, CONVERT_BUFSIZE, "%s", utf8buf);
			p += i;
			rest -= i;
		}
	}
	*p = '\0';

	return pr;
}

void
debug_print_event(IIIMCF_event ev)
{
	IIIMF_status st;
	IIIMCF_event_type et;

	st = iiimcf_get_event_type(ev, &et);
	if(st != IIIMF_STATUS_SUCCESS) {
		fprintf(stderr, "CANNOT_GET_EVENT_TYPE\n");
		return;
	}

	switch(et) {
		case IIIMCF_EVENT_TYPE_DESTROY:
			fprintf(stderr, "EVENT_TYPE_DESTROY\n");
			break;
		case IIIMCF_EVENT_TYPE_RESET:
			fprintf(stderr, "EVENT_TYPE_RESET\n");
			break;
		case IIIMCF_EVENT_TYPE_KEYEVENT:
			fprintf(stderr, "EVENT_TYPE_KEYEVENT\n");
			break;
		case IIIMCF_EVENT_TYPE_KEYEVENT_END:
			fprintf(stderr, "EVENT_TYPE_KEYEVENT_END\n");
			break;
		case IIIMCF_EVENT_TYPE_TRIGGER_NOTIFY:
			fprintf(stderr, "EVENT_TYPE_TRIGGER_NOTIFY\n");
			break;
		case IIIMCF_EVENT_TYPE_TRIGGER_NOTIFY_END:
			fprintf(stderr, "EVENT_TYPE_TRIGGER_NOTIFY_END\n");
			break;
		case IIIMCF_EVENT_TYPE_OPERATION:
			fprintf(stderr, "EVENT_TYPE_OPERATION\n");
			break;
		case IIIMCF_EVENT_TYPE_SETICFOCUS:
			fprintf(stderr, "EVENT_TYPE_SETICFOCUS\n");
			break;
		case IIIMCF_EVENT_TYPE_UNSETICFOCUS:
			fprintf(stderr, "EVENT_TYPE_UNSETICFOCUS\n");
			break;
		case IIIMCF_EVENT_TYPE_EVENTLIKE_END:
			fprintf(stderr, "EVENT_TYPE_EVENTLIKE_END\n");
			break;
		case IIIMCF_EVENT_TYPE_UI:
			fprintf(stderr, "EVENT_TYPE_UI\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_PREEDIT:
			fprintf(stderr, "EVENT_TYPE_UI_PREEDIT\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_PREEDIT_START:
			fprintf(stderr, "EVENT_TYPE_UI_PREEDIT_START\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_PREEDIT_CHANGE:
			fprintf(stderr, "EVENT_TYPE_UI_PREEDIT_CHANGE\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_PREEDIT_DONE:
			fprintf(stderr, "EVENT_TYPE_UI_PREEDIT_DONE\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_LOOKUP_CHOICE:
			fprintf(stderr, "EVENT_TYPE_UI_LOOKUP_CHOICE\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_LOOKUP_CHOICE_START:
			fprintf(stderr, "EVENT_TYPE_UI_LOOKUP_CHOICE_START\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_LOOKUP_CHOICE_CHANGE:
			fprintf(stderr, "EVENT_TYPE_UI_LOOKUP_CHOICE_CHANGE\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_LOOKUP_CHOICE_DONE:
			fprintf(stderr, "EVENT_TYPE_UI_LOOKUP_CHOICE_DONE\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_STATUS_START:
			fprintf(stderr, "EVENT_TYPE_UI_STATUS_START\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_STATUS_CHANGE:
			fprintf(stderr, "EVENT_TYPE_UI_STATUS_CHANGE\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_STATUS_DONE:
			fprintf(stderr, "EVENT_TYPE_UI_STATUS_DONE\n");
			break;
		case IIIMCF_EVENT_TYPE_UI_COMMIT:
			fprintf(stderr, "EVENT_TYPE_UI_COMMIT\n");
			break;
		case IIIMCF_EVENT_TYPE_AUX_START:
			fprintf(stderr, "EVENT_TYPE_AUX_START\n");
			break;
		case IIIMCF_EVENT_TYPE_AUX_DRAW:
			fprintf(stderr, "EVENT_TYPE_AUX_DRAW\n");
			break;
		case IIIMCF_EVENT_TYPE_AUX_SETVALUES:
			fprintf(stderr, "EVENT_TYPE_AUX_SETVALUES\n");
			break;
		case IIIMCF_EVENT_TYPE_AUX_DONE:
			fprintf(stderr, "EVENT_TYPE_AUX_DONE\n");
			break;
		default:
			fprintf(stderr, "UNKNOWN_EVENT\n");
			break;
	}

	return;
}
